% Black-Derman-Toy interest rate tree fitted to yield curve data.
% Shows that zero-coupon bond market prices are recovered, in the 
% sense that a zero-coupon bond priced off the tree 
% will match the market price of a zero-coupon bond.

clc; clear;

% Volatility
sigma = .19;

% Tenors, rates.
Tenors = 1:12;
Rates  = [4.6 4.75 5.0 5.2 5.5 5.81 6.10 6.43 6.9 7.1 7.25 7.49 ]./100;

% Number of time periods (N) and maturity.
N = length(Tenors)-1;
T = N;
dt = T/N;

% Zero coupon bond prices
P = 1./(1+Rates.*dt).^Tenors;

% Return the interest rate tree (r), the discount factors (d),
% and the Arrow-Debreu securities prices
[r d Q] = BuildBDTtree_yields(N,Rates(1),P,sigma,dt);

% Calculate the zero coupon bond prices from the tree.
for m=1:N
	Bond(m) = BDTDiscountBond(m,N,d);
end

% Compare the tree prices to those from the market.
% They are the same.
Comparison = [Bond' P(1:end-1)']

