function [r d Q] = BuildBDTtree_yields(N,Rate,P,sigma,dt);

% Construct the Black-Derman-Toy interest rate tree,
% fitted to yield curve data only.
% Inputs are:
% N     = Number of time steps (scalar)
% Rate  = Zero-coupon bond rates at time zero (scalar).
% P     = Discount factors (N+1) vector.
% sigma = Volatility (scalar).
% dt    = Time increment (scalar).


% Intialize the required matrices
d = zeros(2*N+1,N);
r = zeros(2*N+1,N);
Q = zeros(2*N+1,N);
J = [N:-1:-N];

% Build the Black-Derman-Toy tree.
for j=1:N+1;
	if j==1  % First column
		Q(N+1,1) = 1;
		U(1) = Rate;
		r(N+1,1) = Rate;
		d(N+1,1) = 1/(1+r(N+1,1)*dt);
	else
		for i=N-j+2:2:N+j
			if i==N-j+2        % First row
				Q(i,j) = 0.5*Q(i+1,j-1)*d(i+1,j-1);
			elseif i==N+j      % Last row
				Q(i,j) = 0.5*Q(i-1,j-1)*d(i-1,j-1);
			else               % Middle rows
				Q(i,j) = 0.5*Q(i-1,j-1)*d(i-1,j-1) + 0.5*Q(i+1,j-1)*d(i+1,j-1);
			end
		end
		I = [N-j+2:2:N+j];
		U(j) = fminsearch(@(u) findU_yields(u,Q(I,j),sigma,J(I),dt,P(j)), 0);
		r(I,j) = U(j).*exp(sqrt(dt).*J(I).*sigma);
		d(I,j) = 1./(1+r(I,j).*dt);
	end
end
